#this fine is a workflow in R to conduct a complete Effect Direction MEta-analysis of GWAS.#---go to the work directory of the edme.demo(you will need to use your own path (!!!YOURPATH!!!) to the edme.demo file)
setwd('/!!YOURPATH!!/edme.demo')
#---these three libraries are need for this analysis
library(data.table)
library(dplyr)
library(qvalue)
#---define two paths to GWAS results from two populations
path1 <-  paste0('pop1/')
path2 <-  paste0('pop2/')
#---read two files for sign comparison using the script
#script read2pgwa.R gives the function of 'read2pgwa'
#source('read2pgwa.R')
source('read2pgwaOS1.1.R')
#-necessary arguments of read2pgwa():
#path1: the path to the GWAS results of population 1
#path2: the path to the GWAS results of population 2
#id: the column name in the GWAS results give the ID of SNPs; in this case the id is 'SNP'
#fpa: file suffix: the suffix of the GWAS results', in this case the suffix is 'mlma', which is the standard GWAS output of GCTA
#===note: the function read2pgwaOS() works on the assumption that there is no sign flip due to technical issues, i.e., the A1 of GWAS of the population 1 is the same from the A1 of GWAS of the population 2. Also, read2pgwaOS() assumes that the order of rows (SNP) are the same in different GWAS files. If these two assumptions are not met, please use source('read2pgwa.R') which will do the correction and sorting. However, this will make the reading process slow if you have large datasets.
#then read in the GWAS results for population 1 and population 2
dlist <- read2pgwaOS(path1,path2,id='SNP',fpa='mlma')
#==for unorganised data (effect alleles and SNP rows are not necessarily the same between traits/populations)
#source('read2pgwa.R')
#dlist <- read2pgwa(path1,path2,id='SNP',fpa='mlma')
#----calculate single-trait and multi-trait FDRed with EDME
source('edmesc.v1.3.R')
#--Use all SNPs to do EDME within a range of p-value thresholds:
#-do or do not impose the p-value threshold in the two populations, therefore each population will have a set of FDRed and conventional FDR
#necessary arguments:
#dlist: the read-in files of GWAS results by read2pgwaOS() or read2pgwa()
#k: the number of traits included for analysis
#id: the column name of the marker ID
#chr: the column name of the marker chromosome
#bp: the column name of the marker bp position
#bpa: column name of the marker effect (beta)
#sepa: column name of the marker effect standard error (se)
#ppa: column name of the marker effect p-value (se)
#pvec: a vector of p value thresholds that you want to calculate FDR (e.g., c(0.1,0.01,0.001,...))
#cau: if the p value threshold is only imposed in 1 population not both, =F: Not imposed in two populations; =T: imposed in both populations.
scdt.all <- edmesc(dlist,k=5,id='SNP',chr='Chr',bp='bp',bpa='b',sepa='se',ppa='p',pvec=c(10^(0:-20)),cau=F)
#do not impose the p-value threshold in two populations, therefore each population will have a set of FDRed and conventional FDR

#impose the p-value threshold in two populations, the FDRed for two populations will be the same
scdt.cau <- edmesc(dlist,k=5,id='SNP',chr='Chr',bp='bp',bpa='b',sepa='se',ppa='p',pvec=c(10^(0:-20)),cau=T)

#edmesc() will produce three results in a list
names(scdt.cau)
#[1] "sfdred" "mcount" "mfdred"

#---the 1st result is the single-trait FDRed and conventional FDR (Bolormaa 2014); the column from left right:
head(scdt.all[[1]])
#tr: order of trait
#cut: p-value threshold for population 1
#same.r: the number of variants with the same direction between populations at the p-value threshold for population 1
#t0.r: the number of variants true 0 effect, which is equivalent to the number of variants with different directions between populations at the p-value threshold for population 1 times 2 ((ti.r-same.r)*2)
#ti.r< the total number of significant variants at the p-value threshold for population 1
#fdr.ed.r: FDRed (t0.r/ti.r) at the p-value threshold for population 2
#fdr.r: conventional FDR at the p-value threshold for population 2
#t0.v: the number of variants true 0 effect, which is equivalent to the number of variants with different directions between populations at the p-value threshold for population 2 times 2 ((ti.r-same.r)*2)
#ti.v< the total number of significant variants at the p-value threshold for population 2
#fdr.ed.v: FDRed (t0.r/ti.r) at the p-value threshold for population 2
#fdr.v: conventional FDR at the p-value threshold for population 2
#==if you used cau=T, then the p-value threshold is imposed in both population and the 1st result will only have one set of FDRs.
head(scdt.cau[[1]])

#---the 2nd result is the count of the agreement of variant effect at the variant level. For each variant across all traits analysed, 1 means this variant has agreed variant effect directions between two populations and 0 means this variant has disagreed variant effect directions between two populations
head(scdt.all[[2]])

#---the 3rd result is the summary of the 2nd result for each variant across all traits.
#csd.all: number of traits that this variant had agreed effects between populations
#mt0: estimated proportion of true zero effects (in this case =(1/3)*2)
#mtr: estimated proportion of true effects (1-mt0)
#trn: estimated number of true effects (round(mtr*k,0); this is the 'predicted' number of pleiotropy which is imprecise for each variant, but is a good estimated as an average across many variants. However, this is not the multi-trait FDRed which is calculated using the next (dot product) function

#dot product calc between 2 pop:
source('edmedpd.R')
#read in this file will allow you to use the function (edmedpd), which calculates the dot product(pi value) of effects in two populations for each variant. The use of pi is to focus the FDRed on the effects estimated robustly. A positive pi value indicates a variant has overall agreed effects between two populations across traits analysed, and a negative pi indicates that this variant has disagreed effects across two populations. The proportion of variants with negative is the multi-trait FDRed.
mtdpd <- edmedpd(dlist,id='SNP',chr='Chr',bp='bp',bpa='b',sepa='se')
head(mtdpd)
#if you have large datasets, you should save these multi-trait meta-analysis results before the next analysis
#write.table(scdt.cau$sfdred,gzfile('res/sfdred.cau.34ct.txt.gz'),row.names=F,quote=F)
#write.table(scdt.cau$mfdred,gzfile('res/mfdred.cau.34ct.txt.gz'),row.names=F,quote=F)
#write.table(scdt.cau$mcount,gzfile('res/mcount.cau.34ct.txt.gz'),row.names=F,quote=F)
#write.table(mtdpd,gzfile('res/mtpi.txt.gz'),row.names=F,quote=F)

#--calculate the traditional multi-trait p-value; the script spmt2.R is an R script to do the traditional multi-trait meta-analysis by Bolormaa et al 2014:
source('spmt2.R')
pop1.mt <- etme(dlist[[1]],id='SNP',chr='Chr',bp='bp',bpa='b',sepa='se')
colnames(pop1.mt)[4:ncol(pop1.mt)] <- paste0('pop1.',colnames(pop1.mt)[4:ncol(pop1.mt)])
pop2.mt <- etme(cbind(dlist[[2]][,1],dlist[[1]][,2:3],dlist[[2]][,-1]),id='SNP',chr='Chr',bp='bp',bpa='b',sepa='se')
colnames(pop2.mt)[4:ncol(pop2.mt)] <- paste0('pop2.',colnames(pop2.mt)[4:ncol(pop2.mt)])
#if you have large datasets, you should save these multi-trait meta-analysis results
#write.table(pop1.mt,gzfile('res/pop1.meta.res.txt.gz'),row.names=F,quote=F)
#write.table(pop2.mt,gzfile('res/pop2.meta.res.txt.gz'),row.names=F,quote=F)

#===use the multi-trait p-value in combination with the pi value to calculate the multi-trait FDRed
mtdt <- Reduce(function(...) merge(...,by='SNP',all=T,sort=F),list(mtdpd,scdt.all$mfdred,pop1.mt[,c(1,6)],pop2.mt[,c(1,6)]))
#--calculate multi-trait fdr when the multi-trait p-value is imposed in two populations, also calculate the predicted number of True Effects which will be used in the next step to calculated the observed number of  True effects
  plist <- list()
for (p in c(1,0.5,0.1,5e-2,1e-3,5e-4,1e-6,5e-8,5e-10)){
  plist[[as.character(p)]] <- mtdt[pop1.X2.p<p&pop2.X2.p<p][,list(p,.N,mean(csd.all,na.rm=T),sd(csd.all,na.rm=T)/sqrt(.N),mean(trn,na.rm=T),sd(trn,na.rm=T)/sqrt(.N),sum(pi<=0,na.rm=T)*2/.N,p*(1- .N/mtdt[,.N]) / ((.N/mtdt[,.N])*(1-p)))]
}
  pdt <- setDT(data.frame(do.call(rbind,plist)))
  colnames(pdt) <- c('pthresh','N','N.tr.sameDir','N.tr.sameDir.se','predict.TE','predict.TE.se','FDRed','convent.FDR')
  pdt[,FDRed:=ifelse(FDRed>=1,1,FDRed)]
  pdt[,convent.FDR:=ifelse(!is.infinite(convent.FDR)&convent.FDR>=1,1,convent.FDR)]
print(pdt,nrow(pdt))

#===calculate the observed number of True Effects
#--this calculation uses the number of the predicted TE and the number of significant  variants at a given p-value. Here we use the weighted multi-trait p-value (Xiang et al 2018) from two populations for the calculation. The function is based on 'wt.R' to calculate a weighted t value based on two populations
source('wt.R')
wtlist <- w2t(dlist,id='SNP',chr='Chr',bp='bp',bpa='b',sepa='se')
#save weighted t
#write.table(wtlist$bcwt,gzfile('res/chol.tw.txt.gz'),row.names=F,quote=F)
#---then we use the etme function (traditional multi-trait meta-analysis in Bolormaa et al 2014) to calculate the weighted multi-trait p-value for each variant
wmt <- etme(wtlist$bcwbse,id='SNP',chr='Chr',bp='bp',bpa='wb',sepa='wse')
#save weighted meta-results
#write.table(wmt,gzfile('res/bc.meta.res.txt.gz'),row.names=F,quote=F)

#--then choose a p-value to count the number of variants that are significant
cut<- 1e-6
#a cutoff of p<1e-6 is used on our manuscript
n.wsig <- wmt[X2.p<cut,.N]
#--then the predicted combination of variant-trait is:
predicted.CbT <- n.wsig*round(pdt[pthresh==cut]$predict.TE,0)
predicted.CbT

#--this means that 'predicted.CbT' trait by variant combinations would be important, then we use this number as a cutoff to rank the weighted t-values of variants
wtval <- wtlist$bcwt
wtval.m <- melt(wtval[,c(1,4:ncol(wtval)),with=F],id.var='SNP')
wtval.m[,rank:=frank(-abs(value))]
selected.snp <- wtval.m[rank<predicted.CbT]

#----from here the analysis is done
#-you can extract two information from here:
#1. observed number of traits associated with each variant
ob.te <- selected.snp[,.N,by=SNP]
ob.te[,summary(N)]
#we observe that each variant affected around 3 traits at the weighted multi-trait p=1e-6 level
#2. you can also extract the variants associated with each trait from the table. For example, the SNPs associated with tr1 is:
selected.snp[variable=='tr1.tw']

#====end of analysis =====

